import moviepy.editor as mp
import pathlib
from typing import List, Optional
from datetime import datetime
from functools import lru_cache


@lru_cache()
def datetime_str() -> str:
    return datetime.now().strftime("%Y-%m-%d_%H-%M-%S")


def create_video_clips(
    input_video_folders_path: pathlib.Path, max_n_videos: Optional[int] = None
) -> List[mp.VideoFileClip]:
    VIDEO_EXTENSIONS = ["*.mp4", "*.webm", "*.avi", "*.mov", "*.mkv", "*.gif"]

    video_file_paths = sum(
        [
            list(input_video_folders_path.glob(video_extension))
            for video_extension in VIDEO_EXTENSIONS
        ],
        [],
    )

    max_n_videos = max_n_videos if max_n_videos is not None else len(video_file_paths)
    selected_video_file_paths = video_file_paths[:max_n_videos]

    video_clips = [
        mp.VideoFileClip(str(video_file_path))
        for video_file_path in selected_video_file_paths
    ]
    return video_clips


def overlay_text_on_clip(
    clip: mp.VideoFileClip,
    text: str,
    vertical_position: str = "top",
    horizontal_position: str = "center",
    color: str = "black",
    bg_color: str = "white",
    width_buffer: float = 0.9,
) -> mp.VideoFileClip:
    LARGE_FONT_SIZE_FOR_BETTER_RESOLUTION = 100
    text_clip = (
        mp.TextClip(
            text,
            color=color,
            bg_color=bg_color,
            fontsize=LARGE_FONT_SIZE_FOR_BETTER_RESOLUTION,
        )
        .set_position((horizontal_position, vertical_position))
        .set_duration(clip.duration)
        .resize(width=int(clip.w * width_buffer))
    )
    return mp.CompositeVideoClip([clip, text_clip])



from tap import Tap
from typing import Optional, List
import moviepy.editor as mp
import pathlib
import math
from video_montage.utils import create_video_clips, datetime_str, overlay_text_on_clip


class ArgumentParser(Tap):
    input_video_folder_path: pathlib.Path
    output_video_folder_path: pathlib.Path = pathlib.Path(".")
    output_video_filename: str = f"{datetime_str()}_video_grid_montage.mp4"
    max_n_videos: Optional[int] = None
    max_duration_seconds: Optional[int] = None
    fps: Optional[int] = None
    num_per_row: Optional[int] = None
    overlay_filename: bool = False


def trim_video_clips(
    video_clips: List[mp.VideoFileClip], max_duration_seconds: Optional[int] = None
) -> List[mp.VideoFileClip]:
    shortest_duration_seconds = min([video_clip.duration for video_clip in video_clips])
    montage_duration_seconds = (
        min(shortest_duration_seconds, max_duration_seconds)
        if max_duration_seconds is not None
        else shortest_duration_seconds
    )
    video_clips = [
        video_clip.subclip(0, montage_duration_seconds) for video_clip in video_clips
    ]
    return video_clips


def main() -> None:
    args: ArgumentParser = ArgumentParser().parse_args()
    assert (
        args.input_video_folder_path.exists()
    ), f"Input video folder path {args.input_video_folder_path} does not exist."

    if not args.output_video_folder_path.exists():
        print(f"Creating output folder {args.output_video_folder_path}")
        args.output_video_folder_path.mkdir(parents=True)

    video_clips = create_video_clips(
        args.input_video_folder_path, max_n_videos=args.max_n_videos
    )
    video_clips = trim_video_clips(
        video_clips, max_duration_seconds=args.max_duration_seconds
    )
    num_videos = len(video_clips)

    if args.overlay_filename:
        video_clips = [
            overlay_text_on_clip(
                clip=video_clip, text=pathlib.Path(video_clip.filename).name
            )
            for video_clip in video_clips
        ]

    DEFAULT_NUM_PER_ROW = math.ceil(math.sqrt(num_videos))
    num_per_row = (
        args.num_per_row if args.num_per_row is not None else DEFAULT_NUM_PER_ROW
    )
    num_rows = math.ceil(num_videos / num_per_row)

    num_slots = num_per_row * num_rows
    num_empty_slots = num_slots - num_videos
    BLACK = (0, 0, 0)
    video_clips += [
        mp.ColorClip(
            size=video_clips[0].size,
            color=BLACK,
            duration=video_clips[0].duration,
        )
        for _ in range(num_empty_slots)
    ]
    assert len(video_clips) == num_slots, f"{len(video_clips)} != {num_slots}"

    video_grid = mp.clips_array(
        [video_clips[i * num_per_row : (i + 1) * num_per_row] for i in range(num_rows)]
    )

    output_filepath = args.output_video_folder_path / args.output_video_filename
    fps = args.fps if args.fps is not None else video_clips[0].fps
    video_grid.write_videofile(
        str(output_filepath),
        fps=fps,
    )


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages
from pathlib import Path

VERSION = "0.0.3"
DESCRIPTION = "Easily create a video montage (either a sequence or a grid of videos)"
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="video_montage",
    version=VERSION,
    author="Tyler Lum",
    author_email="tylergwlum@gmail.com",
    url="https://github.com/tylerlum/video_montage",
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=["moviepy", "typed-argument-parser"],
    keywords=["python", "video", "montage", "grid", "sequence"],
    entry_points={
        "console_scripts": [
            "video_grid_montage=video_montage.video_grid_montage:main",
            "video_sequence_montage=video_montage.video_sequence_montage:main",
        ],
    },
    classifiers=[
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
    ],
)



from tap import Tap
from typing import Optional
import moviepy.editor as mp
import pathlib
from video_montage.utils import create_video_clips, datetime_str, overlay_text_on_clip


class ArgumentParser(Tap):
    input_video_folder_path: pathlib.Path
    output_video_folder_path: pathlib.Path = pathlib.Path(".")
    output_video_filename: str = f"{datetime_str()}_video_sequence_montage.mp4"
    max_n_videos: Optional[int] = None
    fps: Optional[int] = None
    overlay_filename: bool = False


def main() -> None:
    args: ArgumentParser = ArgumentParser().parse_args()
    assert (
        args.input_video_folder_path.exists()
    ), f"Input video folder path {args.input_video_folder_path} does not exist."

    if not args.output_video_folder_path.exists():
        print(f"Creating output folder {args.output_video_folder_path}")
        args.output_video_folder_path.mkdir(parents=True)

    video_clips = create_video_clips(
        args.input_video_folder_path, max_n_videos=args.max_n_videos
    )

    if args.overlay_filename:
        video_clips = [
            overlay_text_on_clip(
                clip=video_clip, text=pathlib.Path(video_clip.filename).name
            )
            for video_clip in video_clips
        ]

    video_sequence = mp.concatenate_videoclips(video_clips, method="compose")

    output_filepath = args.output_video_folder_path / args.output_video_filename
    fps = args.fps if args.fps is not None else video_clips[0].fps
    video_sequence.write_videofile(
        str(output_filepath),
        fps=fps,
    )


if __name__ == "__main__":
    main()
